macroscript RegionRenderPRO
category:"BodyulCG"
tooltip:"Region Render PRO"
(	
	fn round v = 
		return floor (v + 0.5)
	
	fn getViewSafeFrameRect =
	(
		local viewportSize = getViewSize()
		local vRatio = viewportSize.x / viewportSize.y
		local renderSize = [renderwidth, renderheight]
		local rRatio = renderSize.x / renderSize.y

		local rect = box2 0 0 0 0
			
		if vRatio <= rRatio or vRatio > rRatio and not displaySafeFrames then
		(
			local s = renderSize.x / viewportSize.x
			
			rect.x = 0
			rect.y = viewportSize.y / 2.0 - renderSize.y / s / 2.0
			rect.w = viewportSize.x
			rect.h = round (renderSize.y / s)
		)
		else
		(
			local s = renderSize.y / viewportSize.y
			
			rect.x = viewportSize.x / 2.0 - renderSize.x / s / 2.0
			rect.y = 0
			rect.w = round (renderSize.x / s)
			rect.h = viewportSize.y
		)
		
		return rect
	)
		
	fn getViewRegionRect =
	(
		local regionRect = viewport.getRegionRect viewport.activeViewport
		local safeFrameRect = getViewSafeFrameRect()
		
		local x = safeFrameRect.x + round (safeFrameRect.w * (float regionRect.x / renderwidth))
		local y = safeFrameRect.y + round (safeFrameRect.h * (float regionRect.y / renderheight))
		local w = round (safeFrameRect.w * (float regionRect.w / renderwidth))
		local h = round (safeFrameRect.h * (float regionRect.h / renderheight))
		
		return box2 x y w h
	)

	rollout RegionFrameBuffer ""
	(
		imgtag img pos:[0, 0] width:RegionFrameBuffer.width height:RegionFrameBuffer.height
		dotnetcontrol pnlBackground "System.Windows.Forms.Panel" pos:[0, 0] width:RegionFrameBuffer.width height:RegionFrameBuffer.height
		
		fn setBitmap bitmap =
		(
			pnlBackground.hide()
			img.bitmap = bitmap
		)
		
		on RegionFrameBuffer open do
		(
			pnlBackground.backcolor = (dotnetclass "System.Drawing.Color").fromArgb 255 35 35 35
			
			local lblStatus = dotnetobject "System.Windows.Forms.Label"
			lblStatus.text = "Rendering ..."
			lblStatus.location = dotnetobject "System.Drawing.Point" 8 8
			lblStatus.size = dotnetobject "System.Drawing.Size" 100 18
			lblStatus.backcolor = (dotnetclass "System.Drawing.Color").fromArgb 255 35 35 35
			lblStatus.forecolor = (dotnetclass "System.Drawing.Color").fromArgb 255 240 240 240
			lblStatus.font = dotnetobject "System.Drawing.Font" lblStatus.font.fontfamily 10
			pnlBackground.controls.add lblStatus
			
			local lblInfo = dotnetobject "System.Windows.Forms.Label"
			lblInfo.text = "Press ESC to cancel"
			lblInfo.location = dotnetobject "System.Drawing.Point" 8 28
			lblInfo.size = dotnetobject "System.Drawing.Size" 130 16
			lblInfo.backcolor = (dotnetclass "System.Drawing.Color").fromArgb 255 35 35 35
			lblInfo.forecolor = (dotnetclass "System.Drawing.Color").fromArgb 255 175 175 175
			pnlBackground.controls.add lblInfo
		)
		
		on img mousedown do 
			destroyDialog RegionFrameBuffer
		
		on RegionFrameBuffer close do
			if img.bitmap != undefined then
			(
				free img.bitmap
				img.bitmap = undefined
			)
	)	
	
	fn getActiveViewScreenPos =
	(
		local p = [0, 0]
		
		if (maxVersion())[1] >= 16000 then
		(
			local b2 = windows.getWindowPos (viewport.getHWND()) -- return box2 x y w h
			
			p.x = b2.x
			p.y = b2.y
		)
		else
		(
			p.x = mouse.screenpos.x - mouse.pos.x
			p.y = mouse.screenpos.y - mouse.pos.y
			
			-- if mouse.pos.x < 0 then p.x += 1;
			-- if mouse.pos.y < 0 then p.y += 1;
		)
		
		return p
	)
	
	fn regionRender resolution:#full alpha:false = 
	(	
		local viewPos = getActiveViewScreenPos()
		local safeFrame = getViewSafeFrameRect()
		local regionRect = getViewRegionRect()
		
		local x = viewPos.x + regionRect.x
		local y = viewPos.y + regionRect.y
		local w = regionRect.w
		local h = regionRect.h
		
		destroyDialog RegionFrameBuffer
		createDialog RegionFrameBuffer w h x y bgcolor:black style:#()
		
		local mainFrame = bitmap w h color:black
		local renderImage = bitmap w h color:black
		local wasCancelled
			
		if resolution == #full then
		(
			renderImage = render outputSize:[safeFrame.w, safeFrame.h] rendertype:#regioncrop vfb:off quiet:true cancelled:&wasCancelled
		)
		else
		(			
			local div = case resolution of
			(
				#half: 2;
				#third: 3;
				#quarter: 4;
				default: 1;
			)
			
			local lw = round (float safeFrame.w / div)
			local lh = round (float safeFrame.h / div)
			local lowRes = render outputSize:[lw, lh] rendertype:#regioncrop vfb:off quiet:true cancelled:&wasCancelled

			copy lowRes renderImage
			free lowRes
		)
		
		if wasCancelled then
		(
			destroyDialog RegionFrameBuffer
			free renderImage
		)
		else
		(
			if alpha then
			(
				local viewImage = gw.getViewportDib()
		
				pasteBitmap viewImage mainFrame (box2 regionRect.x regionRect.y viewImage.width viewImage.height) [0, 0]
				pasteBitmap renderImage mainFrame (box2 0 0 renderImage.width renderImage.height) [0, 0] type:#blend
				
				free viewImage
			)
			else
			(
				copy renderImage mainFrame
			)		

			free renderImage
			
			RegionFrameBuffer.setBitmap mainFrame
		)
	)

	rollout RegionRenderPRO "Region Render PRO v2.0" width:200 height:176
	(
		local config = (getdir #plugcfg) + "\RegionRenderPro\config.ini"
		
		button btnRegionToggle "Region toggle" offset:[0, 2] width:186 height:28 align:#center
		label lblAlpha "Alpha:" offset:[-3, 6] align:#left across:2
		checkbox ckbAlpha offset:[8, 6] width:16 align:#right
		label lblResolution "Resolution:" offset:[-3, 4] align:#left across:2
		dropdownlist ddlResolution offset:[4, 0] items:#("Full", "Half", "Third", "Quarter") align:#right
		button btnRender "Render" offset:[0, 6] width:186 height:36 align:#center
		label lblLine "---------------------------------------------" align:#center enabled:false
		hyperlink hpUrl "bodyulcg.com" offset:[0, -4] address:"https://bodyulcg.com" color:(Color 0 180 255) visitedColor:(Color 0 180 255) align:#center
		
		on RegionRenderPro open do
		(
			if doesFileExist config then
			(
				local x = (getIniSetting config "Rollout" "x") as integer
				local y = (getIniSetting config "Rollout" "y") as integer
				
				if x != undefined and y != undefined then
					setDialogPos RegionRenderPro [x, y]
			)
		)
		
		on btnRegionToggle pressed do
		(
			if getRenderType() == #view then
			(
				setRenderType #region
				
				if not EditRenderRegion.IsEditing then
					EditRenderRegion.EditRegion()
				
				renderSceneDialog.update()
			)
			else
			(
				if EditRenderRegion.IsEditing then
				(
					destroyDialog RegionFrameBuffer
					EditRenderRegion.EditRegion()
					setRenderType #view
					renderSceneDialog.update()
				)
				else
				(
					EditRenderRegion.EditRegion()
				)
				
			)
		)
		
		on btnRender pressed do
		(
			regionRender resolution:(ddlResolution.selected as name) alpha:ckbAlpha.checked
			setFocus RegionRenderPro
		)
		
		on RegionRenderPro close do
		(
			if (doesFileExist config) == false then
			(
				local path = getFileNamePath config
				
				if (doesFileExist path) == false then
					makedir path;
			)
			
			local p = getDialogPos RegionRenderPro
			setIniSetting config "Rollout" "x" (int p.x as string)
			setIniSetting config "Rollout" "y" (int p.y as string)
			
			destroyDialog RegionFrameBuffer
		)
	)
	
	on execute do
	(
		if RegionRenderPro.open then 
			destroyDialog RegionRenderPro
		else
			createDialog RegionRenderPro
	)
)